package com.sun.jndi.toolkit.url;

import javax.naming.NameParser;
import javax.naming.Binding;
import javax.naming.NameClassPair;
import javax.naming.NamingEnumeration;
import javax.naming.spi.NamingManager;
import javax.naming.CannotProceedException;
import javax.naming.OperationNotSupportedException;
import java.net.MalformedURLException;
import javax.naming.InvalidNameException;
import javax.naming.CompositeName;
import javax.naming.Name;
import javax.naming.spi.ResolveResult;
import javax.naming.NamingException;
import java.util.Hashtable;
import javax.naming.Context;

public abstract class GenericURLContext implements Context
{
    protected Hashtable<String, Object> myEnv;
    
    public GenericURLContext(Hashtable<?, ?> hashtable) {
        this.myEnv = null;
        this.myEnv = (Hashtable)(Hashtable)((hashtable == null) ? null : hashtable.clone());
    }
    
    @Override
    public void close() throws NamingException {
        this.myEnv = null;
    }
    
    @Override
    public String getNameInNamespace() throws NamingException {
        return "";
    }
    
    protected abstract ResolveResult getRootURLContext(String p0, Hashtable<?, ?> p1) throws NamingException;
    
    protected Name getURLSuffix(String s, String s2) throws NamingException {
        String s3 = s2.substring(s.length());
        if (s3.length() == 0) {
            return new CompositeName();
        }
        if (s3.charAt(0) == '/') {
            s3 = s3.substring(1);
        }
        try {
            return new CompositeName().add(UrlUtil.decode(s3));
        }
        catch (MalformedURLException ex) {
            throw new InvalidNameException(ex.getMessage());
        }
    }
    
    protected String getURLPrefix(String str) throws NamingException {
        int endIndex = str.indexOf(":");
        if (endIndex < 0) {
            throw new OperationNotSupportedException("Invalid URL: " + str);
        }
        ++endIndex;
        if (str.startsWith("//", endIndex)) {
            endIndex += 2;
            int index = str.indexOf("/", endIndex);
            if (index >= 0) {
                endIndex = index;
            }
            else {
                endIndex = str.length();
            }
        }
        return str.substring(0, endIndex);
    }
    
    protected boolean urlEquals(String s, String anObject) {
        return s.equals(anObject);
    }
    
    protected Context getContinuationContext(Name name) throws NamingException {
        Object lookup = this.lookup(name.get(0));
        CannotProceedException cpe = new CannotProceedException();
        cpe.setResolvedObj(lookup);
        cpe.setEnvironment(this.myEnv);
        return NamingManager.getContinuationContext(cpe);
    }
    
    @Override
    public Object lookup(String s) throws NamingException {
        ResolveResult rootURLContext = this.getRootURLContext(s, this.myEnv);
        Context context = (Context)rootURLContext.getResolvedObj();
        try {
            return context.lookup(rootURLContext.getRemainingName());
        }
        finally {
            context.close();
        }
    }
    
    @Override
    public Object lookup(Name name) throws NamingException {
        if (name.size() == 1) {
            return this.lookup(name.get(0));
        }
        Context continuationContext = this.getContinuationContext(name);
        try {
            return continuationContext.lookup(name.getSuffix(1));
        }
        finally {
            continuationContext.close();
        }
    }
    
    @Override
    public void bind(String s, Object o) throws NamingException {
        ResolveResult rootURLContext = this.getRootURLContext(s, this.myEnv);
        Context context = (Context)rootURLContext.getResolvedObj();
        try {
            context.bind(rootURLContext.getRemainingName(), o);
        }
        finally {
            context.close();
        }
    }
    
    @Override
    public void bind(Name name, Object o) throws NamingException {
        if (name.size() == 1) {
            this.bind(name.get(0), o);
        }
        else {
            Context continuationContext = this.getContinuationContext(name);
            try {
                continuationContext.bind(name.getSuffix(1), o);
            }
            finally {
                continuationContext.close();
            }
        }
    }
    
    @Override
    public void rebind(String s, Object o) throws NamingException {
        ResolveResult rootURLContext = this.getRootURLContext(s, this.myEnv);
        Context context = (Context)rootURLContext.getResolvedObj();
        try {
            context.rebind(rootURLContext.getRemainingName(), o);
        }
        finally {
            context.close();
        }
    }
    
    @Override
    public void rebind(Name name, Object o) throws NamingException {
        if (name.size() == 1) {
            this.rebind(name.get(0), o);
        }
        else {
            Context continuationContext = this.getContinuationContext(name);
            try {
                continuationContext.rebind(name.getSuffix(1), o);
            }
            finally {
                continuationContext.close();
            }
        }
    }
    
    @Override
    public void unbind(String s) throws NamingException {
        ResolveResult rootURLContext = this.getRootURLContext(s, this.myEnv);
        Context context = (Context)rootURLContext.getResolvedObj();
        try {
            context.unbind(rootURLContext.getRemainingName());
        }
        finally {
            context.close();
        }
    }
    
    @Override
    public void unbind(Name name) throws NamingException {
        if (name.size() == 1) {
            this.unbind(name.get(0));
        }
        else {
            Context continuationContext = this.getContinuationContext(name);
            try {
                continuationContext.unbind(name.getSuffix(1));
            }
            finally {
                continuationContext.close();
            }
        }
    }
    
    @Override
    public void rename(String str, String str2) throws NamingException {
        String urlPrefix = this.getURLPrefix(str);
        String urlPrefix2 = this.getURLPrefix(str2);
        if (!this.urlEquals(urlPrefix, urlPrefix2)) {
            throw new OperationNotSupportedException("Renaming using different URL prefixes not supported : " + str + " " + str2);
        }
        ResolveResult rootURLContext = this.getRootURLContext(str, this.myEnv);
        Context context = (Context)rootURLContext.getResolvedObj();
        try {
            context.rename(rootURLContext.getRemainingName(), this.getURLSuffix(urlPrefix2, str2));
        }
        finally {
            context.close();
        }
        return;
    }
    
    @Override
    public void rename(Name obj, Name name) throws NamingException {
        if (obj.size() == 1) {
            if (name.size() != 1) {
                throw new OperationNotSupportedException("Renaming to a Name with more components not supported: " + name);
            }
            this.rename(obj.get(0), name.get(0));
        }
        else {
            if (!this.urlEquals(obj.get(0), name.get(0))) {
                throw new OperationNotSupportedException("Renaming using different URLs as first components not supported: " + obj + " " + name);
            }
            Context continuationContext = this.getContinuationContext(obj);
            try {
                continuationContext.rename(obj.getSuffix(1), name.getSuffix(1));
            }
            finally {
                continuationContext.close();
            }
        }
    }
    
    @Override
    public NamingEnumeration<NameClassPair> list(String s) throws NamingException {
        ResolveResult rootURLContext = this.getRootURLContext(s, this.myEnv);
        Context context = (Context)rootURLContext.getResolvedObj();
        try {
            return context.list(rootURLContext.getRemainingName());
        }
        finally {
            context.close();
        }
    }
    
    @Override
    public NamingEnumeration<NameClassPair> list(Name name) throws NamingException {
        if (name.size() == 1) {
            return this.list(name.get(0));
        }
        Context continuationContext = this.getContinuationContext(name);
        try {
            return continuationContext.list(name.getSuffix(1));
        }
        finally {
            continuationContext.close();
        }
    }
    
    @Override
    public NamingEnumeration<Binding> listBindings(String s) throws NamingException {
        ResolveResult rootURLContext = this.getRootURLContext(s, this.myEnv);
        Context context = (Context)rootURLContext.getResolvedObj();
        try {
            return context.listBindings(rootURLContext.getRemainingName());
        }
        finally {
            context.close();
        }
    }
    
    @Override
    public NamingEnumeration<Binding> listBindings(Name name) throws NamingException {
        if (name.size() == 1) {
            return this.listBindings(name.get(0));
        }
        Context continuationContext = this.getContinuationContext(name);
        try {
            return continuationContext.listBindings(name.getSuffix(1));
        }
        finally {
            continuationContext.close();
        }
    }
    
    @Override
    public void destroySubcontext(String s) throws NamingException {
        ResolveResult rootURLContext = this.getRootURLContext(s, this.myEnv);
        Context context = (Context)rootURLContext.getResolvedObj();
        try {
            context.destroySubcontext(rootURLContext.getRemainingName());
        }
        finally {
            context.close();
        }
    }
    
    @Override
    public void destroySubcontext(Name name) throws NamingException {
        if (name.size() == 1) {
            this.destroySubcontext(name.get(0));
        }
        else {
            Context continuationContext = this.getContinuationContext(name);
            try {
                continuationContext.destroySubcontext(name.getSuffix(1));
            }
            finally {
                continuationContext.close();
            }
        }
    }
    
    @Override
    public Context createSubcontext(String s) throws NamingException {
        ResolveResult rootURLContext = this.getRootURLContext(s, this.myEnv);
        Context context = (Context)rootURLContext.getResolvedObj();
        try {
            return context.createSubcontext(rootURLContext.getRemainingName());
        }
        finally {
            context.close();
        }
    }
    
    @Override
    public Context createSubcontext(Name name) throws NamingException {
        if (name.size() == 1) {
            return this.createSubcontext(name.get(0));
        }
        Context continuationContext = this.getContinuationContext(name);
        try {
            return continuationContext.createSubcontext(name.getSuffix(1));
        }
        finally {
            continuationContext.close();
        }
    }
    
    @Override
    public Object lookupLink(String s) throws NamingException {
        ResolveResult rootURLContext = this.getRootURLContext(s, this.myEnv);
        Context context = (Context)rootURLContext.getResolvedObj();
        try {
            return context.lookupLink(rootURLContext.getRemainingName());
        }
        finally {
            context.close();
        }
    }
    
    @Override
    public Object lookupLink(Name name) throws NamingException {
        if (name.size() == 1) {
            return this.lookupLink(name.get(0));
        }
        Context continuationContext = this.getContinuationContext(name);
        try {
            return continuationContext.lookupLink(name.getSuffix(1));
        }
        finally {
            continuationContext.close();
        }
    }
    
    @Override
    public NameParser getNameParser(String s) throws NamingException {
        ResolveResult rootURLContext = this.getRootURLContext(s, this.myEnv);
        Context context = (Context)rootURLContext.getResolvedObj();
        try {
            return context.getNameParser(rootURLContext.getRemainingName());
        }
        finally {
            context.close();
        }
    }
    
    @Override
    public NameParser getNameParser(Name name) throws NamingException {
        if (name.size() == 1) {
            return this.getNameParser(name.get(0));
        }
        Context continuationContext = this.getContinuationContext(name);
        try {
            return continuationContext.getNameParser(name.getSuffix(1));
        }
        finally {
            continuationContext.close();
        }
    }
    
    @Override
    public String composeName(String str, String str2) throws NamingException {
        if (str2.equals("")) {
            return str;
        }
        if (str.equals("")) {
            return str2;
        }
        return str2 + "/" + str;
    }
    
    @Override
    public Name composeName(Name name, Name name2) throws NamingException {
        Name name3 = (Name)name2.clone();
        name3.addAll(name);
        return name3;
    }
    
    @Override
    public Object removeFromEnvironment(String key) throws NamingException {
        if (this.myEnv == null) {
            return null;
        }
        return this.myEnv.remove(key);
    }
    
    @Override
    public Object addToEnvironment(String key, Object value) throws NamingException {
        if (this.myEnv == null) {
            this.myEnv = new Hashtable(11, 0.75f);
        }
        return this.myEnv.put(key, value);
    }
    
    @Override
    public Hashtable<String, Object> getEnvironment() throws NamingException {
        if (this.myEnv == null) {
            return new Hashtable(5, 0.75f);
        }
        return (Hashtable)this.myEnv.clone();
    }
}
